package com.me.pacman.entity;

import com.badlogic.gdx.math.Vector2;
import com.me.pacman.level.LevelTile;
import com.me.pacman.state.LevelState;

public abstract class MovableEntity extends Entity {

    // speed in tiles / second.
    public float speed;
    public boolean moving;
    public Direction currDirection;
    private float tolerance;

    private Direction nextDirection = null;
    public boolean canMove = true;

    public MovableEntity(LevelState state, float x, float y, float speed, boolean moving, Direction currDirection, float turnTolerance) {
        super(state, x, y);
        this.speed = speed;
        this.moving = moving;
        this.currDirection = currDirection;
        this.tolerance = turnTolerance;
    }

    public void update(float dt) {
        super.update(dt);

        if (!moving) {
            return;
        }

        LevelTile nextTile = null;

        if (nextDirection != null) {
            boolean turned = false;
            switch (nextDirection) {
                case UP:
                    nextTile = state.level.getTile(pos.x, pos.y + 1f);
                    if (nextTile == null) {
                        if (currDirection == Direction.DOWN) {
                            turned = true;
                        }
                    } else if (nextTile.isPassable() && Math.abs(pos.x - ((int) pos.x + 0.5f)) <= tolerance) {
                        pos.x = ((int) pos.x) + 0.5f;
                        turned = true;
                    }
                    break;
                case RIGHT:
                    nextTile = state.level.getTile(pos.x + 1f, pos.y);
                    if (nextTile == null) {
                        if (currDirection == Direction.LEFT) {
                            turned = true;
                        }
                    } else if (nextTile.isPassable() && Math.abs(pos.y - ((int) pos.y + 0.5f)) <= tolerance) {
                        pos.y = ((int) pos.y) + 0.5f;
                        turned = true;
                    }
                    break;
                case DOWN:
                    nextTile = state.level.getTile(pos.x, pos.y - 1f);
                    if (nextTile == null) {
                        if (currDirection == Direction.UP) {
                            turned = true;
                        }
                    } else if (nextTile.isPassable() && Math.abs(pos.x - ((int) pos.x + 0.5f)) <= tolerance) {
                        pos.x = ((int) pos.x) + 0.5f;
                        turned = true;
                    }
                    break;
                case LEFT:
                    nextTile = state.level.getTile(pos.x - 1f, pos.y);
                    if (nextTile == null) {
                        if (currDirection == Direction.RIGHT) {
                            turned = true;
                        }
                    } else if (nextTile.isPassable() && Math.abs(pos.y - ((int) pos.y + 0.5f)) <= tolerance) {
                        pos.y = ((int) pos.y) + 0.5f;
                        turned = true;
                    }
                    break;
            }
            if (turned) {
                currDirection = nextDirection;
                nextDirection = null;
            }
        }

        float dist = speed * dt;

        LevelTile currentTile = state.level.getTile(pos.x, pos.y);
        nextTile = null;

        Vector2 new_pos = new Vector2(pos);

        switch (currDirection) {
            case UP:
                if (currentTile == null && pos.y > state.level.height + 1) {
                    new_pos.y = -1f;
                    canMove = true;
                } else {
                    new_pos.y += dist;
                    nextTile = state.level.getTile(new_pos.x, new_pos.y + 0.5f);
                    canMove = nextTile == null || nextTile.isPassable();
                }
                break;
            case RIGHT:
                if (currentTile == null && pos.x > state.level.width + 1) {
                    new_pos.x = -1f;
                    canMove = true;
                } else {
                    new_pos.x += dist;
                    nextTile = state.level.getTile(new_pos.x + 0.5f, new_pos.y);
                    canMove = nextTile == null || nextTile.isPassable();
                }
                break;
            case DOWN:
                if (currentTile == null && pos.y < 0) {
                    new_pos.y = state.level.height + 1;
                    canMove = true;
                } else {
                    new_pos.y -= dist;
                    nextTile = state.level.getTile(new_pos.x, new_pos.y - 0.5f);
                    canMove = nextTile == null || nextTile.isPassable();
                }
                break;
            case LEFT:
                if (currentTile == null && pos.x < 0) {
                    new_pos.x = state.level.width + 1;
                    canMove = true;
                } else {
                    new_pos.x -= dist;
                    nextTile = state.level.getTile(new_pos.x - 0.5f, new_pos.y);
                    canMove = nextTile == null || nextTile.isPassable();
                }
                break;
        }

        // if move isn't going to collide with wall, move normally.
        // otherwise, trim would-be decimal and center entity on tile
        pos.x = canMove? new_pos.x : ((int) new_pos.x) + 0.5f;
        pos.y = canMove? new_pos.y : ((int) new_pos.y) + 0.5f;
    }

    public void setNextDirection(Direction direction) {
        if (direction != currDirection) {
            nextDirection = direction;
        }
    }

    public Direction getNextDirection() {
        return nextDirection;
    }

}
